"""This module contains common calls for the parsers."""

import re, string

from sympy.abc import A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z
from sympy.abc import a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z
from sympy.logic import simplify_logic
from sympy import true, false

def sanitize(someString):
	"""Sanitize the string for eventual inclusion for EmoTracker.
	
	Arguments:
			someString {string} -- The string to sanitize.
	
	Returns:
			string -- The sanitized string.
	"""
	return someString.replace(":", "").replace(" ", "").replace("+", "Plus").replace("(", "").replace(")", "").replace("-", "").replace(".", "").replace("[", "").replace("]", "").replace("&", "And").replace("'", "").replace("!", "not ")

def multireplace(someString, replacements):
	rep_sorted = sorted(replacements, key=len, reverse=True)
	rep_escaped = map(re.escape, rep_sorted)

	pattern = re.compile("|".join(rep_escaped), 0)
	return pattern.sub(lambda match: replacements[match.group(0)], someString)

def simplifyRule(someString):
	allNodes = re.findall(r'(NodeStatusArray\[\"\w+\s?\w+\"\])', someString)
	allNodes = (list(set(allNodes)))
	node_dict = {}
	for num, node in enumerate(allNodes):
		node_dict[node] = string.ascii_letters[num]
	#if ( len(node_dict) == 0 ):
	#	return
	node_dict['or'] = '|'
	node_dict['and'] = '&'
	reverse_dict = {v: k for k, v in node_dict.items()}
	reverse_dict['True'] = 'true'
	reverse_dict['False'] = 'false'
	reducedString = multireplace(someString, node_dict)
	reducedLogical = simplify_logic(reducedString)
	logicalString = multireplace(str(reducedLogical), reverse_dict)
	return logicalString
	# if len(logicalString) <= len(someString):
	# 	return logicalString
	# return someString

def sanitizeRequirementsForIntendedLogic(someString):
	someString = re.sub( 'NodeStatusArray\["(Boost)\w+"\]', 'true', someString)
	someString = re.sub( 'CheckFunctionTable\["(.*?)"\]', 'CheckFunctionTable["\g<1>MinReqs"]', someString)
	someString = re.sub( 'NodeStatusArray\["(.*?)"\]', 'NodeStatusArray["\g<1>MinReqs"]', someString)
	someString = re.sub( 'Tracker:ProviderCountForCode\("Setting(LampGlitch|CatPause|Raindrop|IceRaindrop|PotClip|ObjectZip|ScreenMash|FlarelessExtinction|NoComboKeyFairy)"\)\s>=\s1', 'true', someString)
	someString = re.sub( 'Tracker:ProviderCountForCode\(".*Ammo"\)\s>=\s1', 'true', someString)

	#someString = re.sub( r'NodeStatusArray\["(((Setting)|(Transition))\w+)(MinReqs)"\]', r'NodeStatusArray["\g<1>"]', someString)
	return someString

updateAvailabilityFunction = """
function UpdateAvailability()
	local checking={}
	local to_check=InitialChecks
	local in_initial=true
	while next(to_check)~=nil do
		checking=to_check
		to_check={}
		for k,v in pairs(checking) do
			local real_value=CheckFunctionTable[k]()
			if NodeStatusArray[k] ~= real_value then
				NodeStatusArray[k]=real_value
				if next(NotificationChart[k])~=nil then
					for jk,noti in pairs(NotificationChart[k]) do
						to_check[noti]=true
					end
				end
				if (in_initial==true) and (real_value==false) then
					for to_reset, to_reset_bool in pairs(NodeStatusArray) do
						if string.match(to_reset, \"not%s\") then
							NodeStatusArray[to_reset]=true
						else
							NodeStatusArray[to_reset]=false
						end
					end
					NodeStatusArray["StatePreEscape"]=true
					NodeStatusArray["StatePreEscapeMinReqs"]=true
					to_check=InitialChecks
					break
				end
			end
		end
		in_initial=false
	end
end
"""

updateAvailabilityWithLoggingFunction = """
function UpdateAvailability()
	local checking={}
	local to_check=InitialChecks
	local in_initial=true
	while next(to_check)~=nil do
		checking=to_check
		to_check={}
		for k,v in pairs(checking) do
			real_value=CheckFunctionTable[k]()
			if NodeStatusArray[k] ~= real_value then
				NodeStatusArray[k]=real_value
				print (os.date("%X") .. " " .. k .. \" checked \" .. tostring(real_value))
				if next(NotificationChart[k])~=nil then
					for jk,noti in pairs(NotificationChart[k]) do
						to_check[noti]=true
					end
				end
				if (in_initial==true) and (real_value==false) then
					print (os.date("%X") .. " " .. k .. \" unchecked, resetting all nodes...\")
					for to_reset, to_reset_bool in pairs(NodeStatusArray) do
						if string.match(to_reset, \"not%s\") then
							NodeStatusArray[to_reset]=true
						else
							NodeStatusArray[to_reset]=false
						end
					end
					NodeStatusArray["StatePreEscape"]=true
					NodeStatusArray["StatePreEscapeMinReqs"]=true
					to_check=InitialChecks
					break
				end
			end
		end
		in_initial=false
	end
end
"""

updateEntranceAvailabilityWithLoggingFunction = """
function UpdateAvailability()
	local checking={}
	local to_check=InitialChecks
	local in_initial=true
	while next(to_check)~=nil do
		checking=to_check
		to_check={}
		for k,v in pairs(checking) do
			real_value=CheckFunctionTable[k]()
			if NodeStatusArray[k] ~= real_value then
				NodeStatusArray[k]=real_value
				print (os.date("%X") .. " " .. k .. \" checked \" .. tostring(real_value))
				if next(NotificationChart[k])~=nil then
					for jk,noti in pairs(NotificationChart[k]) do
						to_check[noti]=true
					end
				end
				if (in_initial==true) and (real_value==false) then
					print (os.date("%X") .. " " .. k .. \" unchecked, resetting all nodes...\")
					for to_reset, to_reset_bool in pairs(NodeStatusArray) do
						if string.match(to_reset, \"not%s\") then
							NodeStatusArray[to_reset]=true
						else
							NodeStatusArray[to_reset]=false
						end
					end
					NodeStatusArray["StatePreEscape"]=true
					NodeStatusArray["StatePreEscapeMinReqs"]=true
					to_check=InitialChecks
					break
				end
			end
		end
		in_initial=false
	end
end
"""
updateEntranceAvailability = """
function UpdateAvailability()
	local checking={}
	local to_check=InitialChecks
	local in_initial=true
	while next(to_check)~=nil do
		checking=to_check
		to_check={}
		for k,v in pairs(checking) do
			real_value=CheckFunctionTable[k]()
			if NodeStatusArray[k] ~= real_value then
				NodeStatusArray[k]=real_value
				if next(NotificationChart[k])~=nil then
					for jk,noti in pairs(NotificationChart[k]) do
						to_check[noti]=true
					end
				end
				if (in_initial==true) and (real_value==false) then
					for to_reset, to_reset_bool in pairs(NodeStatusArray) do
						if string.match(to_reset, \"not%s\") then
							NodeStatusArray[to_reset]=true
						else
							NodeStatusArray[to_reset]=false
						end
					end
					NodeStatusArray["StatePreEscape"]=true
					NodeStatusArray["StatePreEscapeMinReqs"]=true
					to_check=InitialChecks
					break
				end
			end
		end
		in_initial=false
	end
end
"""
updateLocationFunction="""
listofItemizeLocations={"LocationSurfaceMain","LocationSurfaceRuinPathUpper","LocationSurfaceRuinPathLower","LocationGateofGuidanceMain","LocationGateofGuidanceDoor","LocationGateofIllusionEden","LocationGateofIllusionLower","LocationGateofIllusionGrail","LocationGateofIllusionRuin","LocationGateofIllusionPotRoom","LocationMausoleumoftheGiants","LocationGraveyardoftheGiantsWest","LocationGraveyardoftheGiantsGrail","LocationGraveyardoftheGiantsEast","LocationTempleoftheSunTopEntrance","LocationTempleoftheSunMain","LocationTempleoftheSunWest","LocationTempleoftheSunEast","LocationTempleofMoonlightUpper","LocationTempleofMoonlightLower","LocationTempleofMoonlightEden","LocationSpringintheSkyMain","LocationToweroftheGoddessLower","LocationToweroftheGoddessLamp","LocationToweroftheGoddessSpaulder","LocationToweroftheGoddessShieldStatue","LocationInfernoCavernMain","LocationInfernoCavernLava","LocationInfernoCavernViy","LocationInfernoCavernSpikes","LocationTowerofRuinSoutheast","LocationTowerofRuinSouthwestDoor","LocationTowerofRuinIllusion","LocationTowerofRuinMedicine","LocationTowerofRuinTop","LocationChamberofExtinctionMap","LocationChamberofExtinctionMain","LocationChamberofExtinctionLeftMain","LocationChamberofExtinctionMagatamaLeft","LocationChamberofExtinctionMagatamaRight","LocationChamberofExtinctionAnkhUpper","LocationChamberofBirthWestEntrance","LocationChamberofBirthSkanda","LocationChamberofBirthNortheast","LocationChamberofBirthSoutheast","LocationTwinLabyrinthsLoop","LocationTwinLabyrinthsLower","LocationTwinLabyrinthsPoison1","LocationTwinLabyrinthsPoison2","LocationTwinLabyrinthsPoseidon","LocationEndlessCorridor1F","LocationEndlessCorridor2F","LocationEndlessCorridor5F","LocationDimensionalCorridorLower","LocationDimensionalCorridorGrail","LocationShrineoftheMotherMain","LocationShrineoftheMotherLower","LocationShrineoftheMotherSeal","LocationShrineoftheMotherMap","LocationGateofTimeMausoleumLower","LocationGateofTimeMausoleumUpper","LocationGateofTimeGuidance","LocationGateofTimeSurface"}

function reset_locations()
	for k,v in pairs(listofItemizeLocations) do
		Tracker:FindObjectForCode(v).Active=false
	end
end

function update_locations()
	for k,v in pairs(listofItemizeLocations) do
		if (NodeStatusArray["Vanilla"..v]) then
            Tracker:FindObjectForCode(v).Active=true
        end
	end
end
"""