

MuralItem = class(CustomItem)



local MuralData = {

	-- Default
	[0] = {
		["name"] = "BlankMural",
		["code"] = "noMural",
		["image"] = {
			[0] = "images/MuralRando/NoMural.png"
		},
		["overlay"] = "overlay|images/MuralRando/NoMuralOverlay.png"
	},
	
	-- murals
	[1]={
		["name"] = "OriginMural",
		["code"] = "mural1",
		["image"] = {
			[0] = "images/MuralRando/OriginMural.png"
		},
		["overlay"] = "overlay|images/MuralRando/OriginMuralOverlay.png"
	},
	
	[2]={
		["name"] = "BirthMural",
		["code"] = "mural2",
		["image"] = {
			[0] = "images/MuralRando/BirthMural.png"
		},
		["overlay"] = "overlay|images/MuralRando/BirthMuralOverlay.png"
	},
	
	[3]={
		["name"] = "LifeMural",
		["code"] = "mural3",
		["image"] = {
			[0] = "images/MuralRando/LifeMural.png"
		},
		["overlay"] = "overlay|images/MuralRando/LifeMuralOverlay.png"
	},
	
	[4]={
		["name"] = "DeathMural",
		["code"] = "mural4",
		["image"] = {
			[0] = "images/MuralRando/DeathMural.png"
		},
		["overlay"] = "overlay|images/MuralRando/DeathMuralOverlay.png"
	}
}


-- These are all commonly referenced items, three are effectively fixed, DungeonPalette changes based on whichever one is currently selected
local MuralBlank
local MuralCurrentPalette
local MuralRightClick = "overlay|images/MuralRando/NoMuralOverlay.png"
local MuralEmpty = "@disabled"
MuralItem = class(CustomItem)

-- MuralItems are the mutually exclusive items that you use to control what you're putting on a mural square
-- There are no properties used on this one since we do not want this item to change by using undo
-- In addition there is no handling for saving and loading since we want it to return to its default state if a save is loaded

function MuralItem:init(name,code,image,index)
	self:createItem(name)
	self.code = code
	self.Active = false
	self.images = {}
	-- This is mostly so we only have to convert the images to references once
	for i,ref in pairs(image) do
		self.images[i] = ImageReference:FromPackRelativePath(ref)
	end
	self:AssignImage()
	self.index = index
	self.ItemInstance.PotentialIcon = self.activeImage
	
	-- The 0 indexed item is our default state, and also the state our palette should start as
	if index == 0 then
		MuralBlank = self
		MuralCurrentPalette = self
	-- We don't bother caching the blank and right click items, those aren't normally visible so they'll never have to be updated
	end
	
end


function MuralItem:AssignImage()
	self.activeImage = self.images[0]
	self.disabledImage = ImageReference:FromImageReference(self.activeImage, "@disabled")
	self:UpdateIcon()
end

function MuralItem:UpdateIcon()
	if self.Active then
		self.ItemInstance.Icon = self.activeImage
	else
		self.ItemInstance.Icon = self.disabledImage
	end
end

function MuralItem:canProvideCode(code)
    if code == self.code then
        return true
    else
        return false
    end
end

function MuralItem:providesCode(code)
    if code == self.code and self.Active then
        return 1
    end
    return 0
end

function MuralItem:onLeftClick()
	-- Technically this works fine without checking if the item isn't active, but this is cleaner
	if not self.Active then
		MuralCurrentPalette.Active = false
		MuralCurrentPalette:UpdateIcon()
		MuralCurrentPalette = self
		self.Active = true
		-- Since we're not using setProperty we also can't use PropertyChanged() and so have to push the icon update here.
		self:UpdateIcon()
	end
end

function MuralItem:onRightClick()
	-- We do need to double check that the user is right-clicking an active palette item before we set the palette to GridStairsImage
	if self.Active then
		MuralCurrentPalette = MuralBlank
		self.Active = false
		-- Since we're not using setProperty we also can't use PropertyChanged() and so have to push the icon update here
		self:UpdateIcon()
	end
end

-- Since we're not interested in saving palette state, we make the save function return nothing instead of the default empty table
function MuralItem:save()
end

for i,data in pairs(MuralData) do
	local newItem = MuralItem(data["name"], data["code"], data["image"], i)
end
-- MuralLocationItem is the seal that locks something on the map
-- Properties:
-- active - boolean if item is active or not
-- index - integer storing the index of the current Mural, corresponding to its position in MuralItem
-- Variables:
-- self.RightClickImage - The image to be set to when the square is right-clicked
-- self.AssignedImage - The image that the square has currently been set to by left-clicking

MuralLocationItem = class(CustomItem)

function MuralLocationItem:init(name,image)
	self:createItem(name)
    self.code = name
    self.Active=true
	self.DefaultImage = image
	self:setProperty("SealIndicator",0)
	self:setProperty("SealIndicatoractive", 0)
    -- self.RightClickImage = ImageReference:FromPackRelativePath(OnRightClickGridImage)
	self:AssignImage()
	self.ItemInstance.PotentialIcon = self.AssignedImage

end

function MuralLocationItem:getImage()
	-- This looks intimidating but is fairly simple in practice, palette data is effectively laid out in this order:
	-- 1. ID number of the selected palette
	-- 2. Strings defining the different parts of that item; we just care about the image here
	-- 3. Unique graphics for different levels
	-- This function checks if a specific image for the current level exists, and goes to default if it doesn't\
	return ImageReference:FromPackRelativePath(self.DefaultImage,MuralData[self:getIndex()]["overlay"])
end

function MuralLocationItem:AssignImage()
	self.AssignedImage = self:getImage()
	self.ItemInstance.Icon = self.AssignedImage
end


function MuralLocationItem:canProvideCode(code)
    if code == self.code then
        return true
    else
        return false
    end
end

function MuralLocationItem:providesCode(code)
    --always false, using toggle seals to provide codes
    return 0
end

function MuralLocationItem:onLeftClick()
	if self:getIndex() == MuralCurrentPalette.index then
		ToggleSeals[self:getIndex()]:removeCode(self.code)
		self:setProperty("SealIndicator", MuralBlank.index)
	else
		if self:getIndex()~=0 then
			ToggleSeals[self:getIndex()]:removeCode(self.code)
		end
		self:setProperty("SealIndicator", MuralCurrentPalette.index)
		if MuralCurrentPalette.index~=0 then
			ToggleSeals[MuralCurrentPalette.index]:addCode(self.code)
		end
	end
end

function MuralLocationItem:onRightClick()
	local temp = self:getActive()
	temp = temp + 1
	if temp > 2 then
		temp = 0
	end
	self:setProperty("SealIndicatoractive", temp)
end

function MuralLocationItem:propertyChanged(key, value)
	self:AssignImage()
end

function MuralLocationItem:getIndex()
	return self:getProperty("SealIndicator")
end

function MuralLocationItem:getActive()
	return self:getProperty("SealIndicatoractive")
end


function MuralLocationItem:save()
    local saveData = {}
    saveData["index"] = self:getIndex()
    return saveData
end

function MuralLocationItem:load(data)
    if data["index"] ~= nil then
        self:setProperty("SealIndicator",data["index"])
        if data["index"]~=0 then
			ToggleSeals[data["index"]]:addCode(self.code)
		end
    end
    return true
end



local MuralLocationData = {
	{
		["name"] = "seallocationo1",
		["image"]  = "images/MuralRando/SealO1.png"
	},
	{
		["name"] = "seallocationl2",
		["image"]  = "images/MuralRando/SealL2.png"
	},
	{
		["name"] = "seallocationl1",
		["image"]  = "images/MuralRando/SealL1.png"
	},
	{
		["name"] = "seallocationo2",
		["image"]  = "images/MuralRando/SealO2.png"
	},
	{
		["name"] = "seallocationo3",
		["image"]  = "images/MuralRando/SealO3.png"
	},
	{
		["name"] = "seallocationd1",
		["image"]  = "images/MuralRando/SealD1.png"
	},
	{
		["name"] = "seallocationb1",
		["image"]  = "images/MuralRando/SealB1.png"
	},
	{
		["name"] = "seallocationo4",
		["image"]  = "images/MuralRando/SealO4.png"
	},
	{
		["name"] = "seallocationo5",
		["image"]  = "images/MuralRando/SealO5.png"
	},
	{
		["name"] = "seallocationb2",
		["image"]  = "images/MuralRando/SealB2.png"
	},
	{
		["name"] = "seallocationb3",
		["image"]  = "images/MuralRando/SealB3.png"
	},
	{
		["name"] = "seallocationo6",
		["image"]  = "images/MuralRando/SealO6.png"
	},
	{
		["name"] = "seallocationl4",
		["image"]  = "images/MuralRando/SealL4.png"
	},
	{
		["name"] = "seallocationo7",
		["image"]  = "images/MuralRando/SealO7.png"
	},
	{
		["name"] = "seallocationb4",
		["image"]  = "images/MuralRando/SealB4.png"
	},
	{
		["name"] = "seallocationl5",
		["image"]  = "images/MuralRando/SealL5.png"
	},
	{
		["name"] = "seallocationd2",
		["image"]  = "images/MuralRando/SealD2.png"
	},
	{
		["name"] = "seallocationb5",
		["image"]  = "images/MuralRando/SealB5.png"
	},
	{
		["name"] = "seallocationl6",
		["image"]  = "images/MuralRando/SealL6.png"
	},
	{
		["name"] = "seallocationb6",
		["image"]  = "images/MuralRando/SealB6.png"
	},
	{
		["name"] = "seallocationd4",
		["image"]  = "images/MuralRando/SealD4.png"
	},
	{
		["name"] = "seallocationd5",
		["image"]  = "images/MuralRando/SealD5.png"
	},
}


for i,data in pairs(MuralLocationData) do
	local newItem = MuralLocationItem(data["name"], data["image"])
end

